package ca.jamdat.flight;

import ca.jamdat.flight.*;
import ca.jamdat.fuser.*;
import ca.jamdat.flight.String;
import ca.jamdat.flight.Package;


	
		import android.view.KeyEvent;
		import android.view.KeyCharacterMap;
	


public class FlEventQueue {

public final static int EventVoid = 0;
public final static int EventKeyDown = 1;
public final static int EventKeyUp = 2;
public final static int EventFlightKeyDown = 3;
public final static int EventFlightKeyUp = 4;
public final static int EventPause = 5;
public final static int EventResume = 6;
public final static int EventPenMoved = 7;
public final static int EventPenDown = 8;
public final static int EventPenUp = 9;
public final static int EventSizeChanged = 10;

public static FlEventQueue GetInstance () {

	 FrameworkGlobals frameworkGlobals = FrameworkGlobals.GetInstance();

	if(frameworkGlobals.mFlEventQueue == null)
	{
		frameworkGlobals.mFlEventQueue = new FlEventQueue();
	}
	return  frameworkGlobals.mFlEventQueue;

}


public void ProcessEvents () {


	synchronized(FlAndroidApp.paintLock)
	{
		while(this.mEvtCount > 0)
		{
			 int evtType = FlEventQueue.EventVoid;

			 int evtDetailInt = 0;

			short[] evtDetailShort = {0, 0};

			synchronized(this.mEvtQueue)
			{
				
				evtType = (int)(mEvtQueue[0] >>> 32);
				evtDetailInt = (int)(mEvtQueue[0] & 0x00000000FFFFFFFFL);
				evtDetailShort[0] = (short)(evtDetailInt >>> 16);
				evtDetailShort[1] = (short)(evtDetailInt & 0x0000FFFF);

				this.mEvtCount --;

				
				for(int i =0;i <(8-1);i ++)
				{
					mEvtQueue[i] = mEvtQueue[i+1];
				}
				mEvtQueue[8-1] = 0x0000000000000000L;
			}

			switch(evtType)
			{
				case FlEventQueue.EventKeyDown :
				{
					Application.GetInstance().OnKeyFromOS(TranslateKey(evtDetailInt), false);
					break;
				}

				case FlEventQueue.EventKeyUp :
				{
					Application.GetInstance().OnKeyFromOS(TranslateKey(evtDetailInt), true);
					break;
				}

				case FlEventQueue.EventFlightKeyDown :
				{
					Application.GetInstance().OnKeyFromOS(evtDetailInt, false);
					break;
				}

				case FlEventQueue.EventFlightKeyUp :
				{
					Application.GetInstance().OnKeyFromOS(evtDetailInt, true);
					break;
				}

				case FlEventQueue.EventPause :
				{
					FlGameTimer.GetInstance().Pause();
					Application.GetInstance().FocusLost();
					
						SoundManager.Get().StopAllSoundPlayers();
					break;
				}

				case FlEventQueue.EventResume :
				{
					FlGameTimer.GetInstance().Resume();
					Application.GetInstance().FocusGained();
					break;
				}

				
					case FlEventQueue.EventPenMoved :
					{
						 Vector2_short pos = new Vector2_short(evtDetailShort[0], evtDetailShort[1]);

						FlPenManager.Get().OnPenDrag(pos);
						break;
					}

					case FlEventQueue.EventPenDown :
					{
						 Vector2_short pos = new Vector2_short(evtDetailShort[0], evtDetailShort[1]);

						FlPenManager.Get().OnPenDown(pos);
						break;
					}
					
					case FlEventQueue.EventPenUp :
					{
						 Vector2_short pos = new Vector2_short(evtDetailShort[0], evtDetailShort[1]);

						FlPenManager.Get().OnPenUp(pos);
						break;
					}
				case FlEventQueue.EventSizeChanged :
					{
						Application.GetInstance().OnScreenSizeChangedFromOS(evtDetailShort[0], evtDetailShort[1]);
						break;
					}
				}
		}
	}


}


public void AddEvent (int evtType, int evtDetail) {

	
	if(evtType == FlEventQueue.EventPause)
	{
		ClearEventQueue();
	}

	
	 boolean isFirstEventOfPair = (evtType == FlEventQueue.EventKeyDown) || (evtType == FlEventQueue.EventPenDown);



	synchronized(mEvtQueue)
	{
		if(evtType == FlEventQueue.EventResume)
		{
			mIsLocked = false;
		}

		 boolean bustQueue = (this.mEvtCount >= 8) || (isFirstEventOfPair && (this.mEvtCount >= 8-1));


		if(!mIsLocked && !bustQueue)
		{
			
			mEvtQueue[mEvtCount++] = (evtDetail & 0x00000000FFFFFFFFL) | (((long)evtType) << 32);
		}

		if(evtType == FlEventQueue.EventPause)
		{
			mIsLocked = true;
		}

	}


}


public void AddEvent (int evtType, short evtDetail1, short evtDetail2) {

	AddEvent(evtType, evtDetail1 <<16| evtDetail2);

}


public  FlEventQueue () {


	
		mKeyCharacterMap = KeyCharacterMap.load(KeyCharacterMap.BUILT_IN_KEYBOARD);
	for(int i =0;i <8;i ++)
	{
		mEvtQueue[i] = 0x0000000000000000L;
	}

}


public void ClearEventQueue () {


	synchronized(FlAndroidApp.paintLock)
	{
		synchronized(this.mEvtQueue)
		{
			mEvtCount = 0;
			for(int i =0;i <8;i ++)
			{
				mEvtQueue[i] = 0x0000000000000000L;
			}
		}
	}


}


public int TranslateKey (int apiKey) {

	

		
		if(mKeyCharacterMap.isPrintingKey(apiKey))
		{
			int asciiKey;


			if(Application.GetInstance().GetKeyAltMode())
			{
				asciiKey = mKeyCharacterMap.get(apiKey, android.text.method.MetaKeyKeyListener.META_ALT_ON);
			}
			else if(Application.GetInstance().GetKeyShiftMode())
			{
				asciiKey = mKeyCharacterMap.get(apiKey, android.text.method.MetaKeyKeyListener.META_SHIFT_ON);
			}
			else
			{
				asciiKey = mKeyCharacterMap.get(apiKey, android.text.method.MetaKeyKeyListener.META_SYM_ON);
			}

			return  Application.GetInstance().AsciiToFlightKey(asciiKey);
		}

		
		switch(apiKey)
		{
			
			case KeyEvent.KEYCODE_ALT_LEFT :
			case KeyEvent.KEYCODE_ALT_RIGHT :         return  Key.ALT;
			case KeyEvent.KEYCODE_CLEAR :             return  Key.BACKSPACE;
			case KeyEvent.KEYCODE_DEL :               return  Key.BACKSPACE;
			case KeyEvent.KEYCODE_DPAD_DOWN :         return  Key.DOWN;
			case KeyEvent.KEYCODE_DPAD_LEFT :         return  Key.LEFT;
			case KeyEvent.KEYCODE_DPAD_RIGHT :        return  Key.RIGHT;
			case KeyEvent.KEYCODE_DPAD_UP :           return  Key.UP;
			case KeyEvent.KEYCODE_ENTER :             return  Key.ENTER;
			case KeyEvent.KEYCODE_SHIFT_LEFT :
			case KeyEvent.KEYCODE_SHIFT_RIGHT :       return  Key.SHIFT;		

			

case KeyEvent.KEYCODE_DPAD_CENTER :   return  Key.FIRE;
			case KeyEvent.KEYCODE_SOFT_LEFT :     return  Key.SOFT_SELECT;
				case KeyEvent.KEYCODE_SOFT_RIGHT :    return  Key.SOFT_CLEAR;
			


default :					 return  Key.UNDEFINED;
		}
	
	








































































}


public long[] mEvtQueue = new long[8];

public int mEvtCount;
public boolean mIsLocked;
public KeyCharacterMap mKeyCharacterMap;
public void AddEvent(int evtType)
{
	AddEvent(evtType, (int)0);
}

/* One dimension array */
public static FlEventQueue[] InstArrayFlEventQueue(int size) {
	FlEventQueue[] newArray = new FlEventQueue[size];
	for(int i = 0 ; i < size ; i++) newArray[i] = new FlEventQueue();
	return newArray; }

/* Two dimension array */
public static FlEventQueue[][] InstArrayFlEventQueue(int size1, int size2) {
	FlEventQueue[][] newArray = new FlEventQueue[size1][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new FlEventQueue[size2];
		for(int j = 0 ; j < size2 ; j++) newArray[i][j] = new FlEventQueue(); }
	return newArray; }

/* Three dimension array */
public static FlEventQueue[][][] InstArrayFlEventQueue(int size1, int size2, int size3) {
	FlEventQueue[][][] newArray = new FlEventQueue[size1][][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new FlEventQueue[size2][];
		for(int j = 0 ; j < size2 ; j++) {
			newArray[i][j] = new FlEventQueue[size3];
			for(int k = 0 ; k < size3 ; k++) newArray[i][j][k] = new FlEventQueue(); } }
	return newArray; }



}

