package ca.jamdat.flight;

import ca.jamdat.flight.*;
import ca.jamdat.fuser.*;
import ca.jamdat.flight.String;
import ca.jamdat.flight.Package;


public class RigidTransform {

public static RigidTransform Cast(Object o, RigidTransform _) {
 return  (RigidTransform)o; 
}


public void OnSerialize(Package p) {

}


public FVec3T_F32 translation;
public FQuaternionT_F32 rotation;
public byte fixedPointPos;
public RigidTransform (int fixedPointPosition) {
translation = new FVec3T_F32();
rotation = new FQuaternionT_F32();

	fixedPointPos = (byte)fixedPointPosition;

}


public  RigidTransform ( FVec3T_F32 parTranslation, FQuaternionT_F32 parRotation, int fixedPointPosition) {
translation = new FVec3T_F32(parTranslation);
rotation = new FQuaternionT_F32(parRotation);


	fixedPointPos = (byte)fixedPointPosition;

}


public void Init ( FVec3T_F32 parTranslation, FQuaternionT_F32 parRotation, int parFixedPointPosition) {

	translation.Assign(parTranslation);
	rotation.Assign(parRotation);
	fixedPointPos = (byte)parFixedPointPosition;

}


public FVec3T_F32 Transform ( FVec3T_F32 v) {

	return  rotation.Conjugate(v.x, v.y, v.z, fixedPointPos).Add(translation);

}


public void Transform (FVec3T_F32 returnValue, FVec3T_F32 v) {

	returnValue.Assign(rotation.Conjugate(v.x, v.y, v.z, fixedPointPos).Add(translation));

}


public RigidTransform Compose ( RigidTransform t2) {

	return  new RigidTransform(translation.Add(rotation.Conjugate(t2.translation.x, t2.translation.y, t2.translation.z, fixedPointPos)), rotation.Mul(t2.rotation), fixedPointPos);

}


public RigidTransform Inverse () {

	 FQuaternionT_F32 inverseRotation = new FQuaternionT_F32(rotation.GetInverseForNormalized());

	return  new RigidTransform(inverseRotation.Conjugate(translation.Neg().x, translation.Neg().y, translation.Neg().z, fixedPointPos), inverseRotation, fixedPointPos);

}


public void Inverse (RigidTransform returnValue, FQuaternionT_F32 tmpQuaternion) {

	rotation.GetInverseForNormalized(tmpQuaternion);
	returnValue.Init(tmpQuaternion.Conjugate(translation.Neg().x, translation.Neg().y, translation.Neg().z, fixedPointPos), tmpQuaternion, fixedPointPos);

}


public FMat44T_F32 InverseMatrix () {

	return  Inverse().Matrix();

}


public void InverseMatrix (FMat44T_F32 returnMatrix, FMat44T_F32 tmpMatrix1, FMat44T_F32 tmpMatrix2, FQuaternionT_F32 tmpQuaternion, RigidTransform tmpRigidTransform) {

	Inverse(tmpRigidTransform, tmpQuaternion);
	tmpRigidTransform.Matrix(returnMatrix, tmpMatrix1, tmpMatrix2);

}


public FMat44T_F32 Matrix () {

	 FMat44T_F32 trans = new FMat44T_F32(fixedPointPos);

	trans.SetTranslation(translation, fixedPointPos);
	return  trans.Mul(rotation.ToMatrix(fixedPointPos), fixedPointPos);	

}


public void Matrix (FMat44T_F32 returnMatrix, FMat44T_F32 tmpMatrix1, FMat44T_F32 tmpMatrix2) {

	tmpMatrix1.SetTranslation(translation, fixedPointPos);

	rotation.ToMatrix(tmpMatrix2, fixedPointPos);

	tmpMatrix1.Mul(returnMatrix, tmpMatrix2, fixedPointPos);

}


public boolean Equals ( RigidTransform other) {

	return  translation.Equals(other.translation) && rotation.Equals(other.rotation);

}


public RigidTransform Assign ( RigidTransform other) {

	translation.Assign(other.translation);
	rotation.Assign(other.rotation);
	return this;

}


public static RigidTransform I (int fixedPointPos) {

	return  new RigidTransform(FVec3T_F32.Zero(fixedPointPos), FQuaternionT_F32.I(), fixedPointPos);

}


public RigidTransform()
{
	this((int)16);
}

/* One dimension array */
public static RigidTransform[] InstArrayRigidTransform(int size) {
	RigidTransform[] newArray = new RigidTransform[size];
	for(int i = 0 ; i < size ; i++) newArray[i] = new RigidTransform();
	return newArray; }

/* Two dimension array */
public static RigidTransform[][] InstArrayRigidTransform(int size1, int size2) {
	RigidTransform[][] newArray = new RigidTransform[size1][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new RigidTransform[size2];
		for(int j = 0 ; j < size2 ; j++) newArray[i][j] = new RigidTransform(); }
	return newArray; }

/* Three dimension array */
public static RigidTransform[][][] InstArrayRigidTransform(int size1, int size2, int size3) {
	RigidTransform[][][] newArray = new RigidTransform[size1][][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new RigidTransform[size2][];
		for(int j = 0 ; j < size2 ; j++) {
			newArray[i][j] = new RigidTransform[size3];
			for(int k = 0 ; k < size3 ; k++) newArray[i][j][k] = new RigidTransform(); } }
	return newArray; }



}

