package ca.jamdat.tetris;

import ca.jamdat.flight.*;
import ca.jamdat.fuser.*;
import ca.jamdat.flight.String;
import ca.jamdat.flight.Package;


public abstract class BaseScreenManager extends TimeControlled {

public final static byte screenorientationStraight = 0;
public final static byte screenOrientationPortrait = screenorientationStraight;
public final static byte screenOrientationLandscape = screenorientationStraight+1;
public final static byte screenOrientationCount = screenorientationStraight+2;

public final static byte orientationStraight = 0;
public final static byte orientationRight = 1;
public final static byte orientationFlip = 2;
public final static byte orientationLeft = 3;
public final static byte orientationAutomatic = 4;
public final static byte orientationCount = 4;
public final static byte orientationSettingCount = (4 + 1);

public final static byte orientationOperationTurnRight = 0;
public final static byte orientationOperationFlip = 1;
public final static byte orientationOperationTurnLeft = 2;
public final static byte orientationOperationResetToHardwareSetting = 3;
public final static byte orientationOperationCount = 4;

public static int ScreenOrientationToIndex (byte o) {

	
	return  (int)o;

}


public static int OrientationToIndex (byte o) {

	
	return  (int)o;

}


public static byte IndexToOrientation (int i) {

	return  (byte)i;

}


public static TetrisApp GetApp () {

	return  TetrisApp.Get();

}


public static byte GetScreenOrientationForOrientation (byte o) {

	if(o == orientationStraight || o == orientationFlip)
	{
		return  screenorientationStraight;
	}
	else
	{
		return  (byte)(screenorientationStraight + 1);
	}

}


public byte GetDefaultOrientation () {

	byte initialOrientation;



 byte idealOrientation = orientationAutomatic;

	initialOrientation = idealOrientation;
	if(!mOrientationSupported[idealOrientation])
	{
		for(int i = 0;i < orientationSettingCount;i ++)
		{
			 byte ori = (byte)i;

			if(mOrientationSupported[i])
			{
				initialOrientation = ori;
				break;
			}
		}
	}
return  initialOrientation;

}


public BaseScreenManager () {


}


public void Read (FileSegmentStream s) {

	byte ori;


	ori = (byte)s.ReadByte();



Reset(ori);

}


public void Write (FileSegmentStream s) {



s.WriteByte((byte)mOrientationSetting);

}


public void CheckHandleOrientationChange () {

	DoCheckHandleOrientationChange(true);

}


public void SetOrientationSetting (byte o) {




mOrientationSetting = o;

}


public void OnScreenSizeChange () {



	RessourcesManager.Get().RefreshConstants();
	PrepareForScreenSizeChange();
	OnTime();

}


public void OnTime (int a11, int a10) {

	OnTime();

}


public byte GetScreenOrientation () {

	 byte ori = GetOrientation();

	 byte result = GetScreenOrientationForOrientation(ori);

	return  result;

}


public byte GetOrientation () {

	 byte orientationSetting = GetOrientationSetting();

	if(orientationSetting != orientationAutomatic)
	{
		return  orientationSetting;
	}
	else if(IsHardwareRotationEnabled())
	{
		return  GetCurrentHardwareRotation();
	}
	else	
	{
		return  orientationStraight;
	}	

}


public byte GetOrientationSetting () {


	return  mOrientationSetting;



}


public byte GetOrientationOperationResult (byte op) {

	if(op == orientationOperationResetToHardwareSetting)
	{
		return  orientationAutomatic;
	}

	 byte ori = GetOrientation();

	 int resulti = (ori + op + 1) % orientationCount;

	 byte resultOri = (byte)resulti;

	return  resultOri;

}


public boolean IsOrientationSettingNeeded () {

	
	 int supportedOrientationsCount = 0;

	for(int i = 0;i < orientationSettingCount;i ++)
	{
		 byte ori = (byte)i;

		if(IsOrientationSettingSupported(ori))
		{
			supportedOrientationsCount++;
			if(supportedOrientationsCount >= 2)
			{
				return  true;
			}
		}
	}
	return  false;

}


public boolean IsOrientationSettingSupported (byte ori) {




return  mOrientationSupported[ori];

}


public boolean IsNativeOrientationSupported (byte ori) {




return  CalculateIsNativeOrientationSupported(ori);

}


public boolean IsScreenOrientationChanged () {




 byte OrientationSetting = GetOrientationSetting();

	if(OrientationSetting == GetCurrentlyUsedOrientation())
	{
		return  false;
	}
	else
	{
		 byte ori = GetScreenOrientation();

		 byte usedOri = GetScreenOrientationForOrientation(GetCurrentlyUsedOrientation());

		return  ori != usedOri;
	}

}


public boolean IsScreenOrientationChangeHardwareBased () {

	return  GetOrientationSetting() == orientationAutomatic;

}


public void Init () {


	mIsRegisteredInGlobalTime = false;
	mCurrentlyUsingHardwareSetting = true;

	for(int i = 0;i < orientationSettingCount;i ++)
	{
		 byte ori = (byte)i;

		mOrientationSupported[i] = CalculateIsOrientationSettingSupported(ori);
	}

 byte initialOrientation = GetDefaultOrientation();

	if(initialOrientation != orientationAutomatic)
	{
		initialOrientation = GetCurrentHardwareRotation();
	}


	mOrientationSetting = initialOrientation;
Reset(initialOrientation);

}


public void OnTime () {

	OnStartRenderingFrame();
	RefreshGlobalTimeRegistration();

}


public abstract boolean CalculateIsOrientationSettingSupported(byte a11);



public abstract boolean CalculateIsNativeOrientationSupported(byte a12);



public abstract boolean IsHardwareRotationEnabled();



public abstract byte GetCurrentHardwareRotation();



public abstract void SetHardwareOrientation(byte a13);



public void EnsureAppHasProperSize () {


	
	 short screenRect_left = 0;
  short screenRect_top = 0;
  short screenRect_width = (short)(((int)ConstantsEnglishPortrait.screenWidth == (int)ConstantsEnglishLandscape.screenWidth) ? Constants.screenWidth : ConstantsGetter.GetConstants().screenWidthFct());
  short screenRect_height = (short)(((int)ConstantsEnglishPortrait.screenHeight == (int)ConstantsEnglishLandscape.screenHeight) ? Constants.screenHeight : ConstantsGetter.GetConstants().screenHeightFct());

	 Component c = Application.GetInstance();
	c.SetRect(screenRect_left, screenRect_top, screenRect_width, screenRect_height);

}


public boolean IsOrientationChanged () {

	return  GetCurrentlyUsedOrientation() != GetOrientation();

}


public void DoCheckHandleOrientationChange (boolean checkOnTransition) {

	 boolean settingIsAutomatic = GetOrientationSetting() == orientationAutomatic;

	 boolean changedFromAutomaticToSoftware = IsCurrentlyUsingHardwareSetting() != settingIsAutomatic;

	 boolean orientationHasBeenChanged = GetOrientation() != GetCurrentlyUsedOrientation();


	boolean settingHasBeenChanged;

	settingHasBeenChanged = changedFromAutomaticToSoftware ||
			(!IsCurrentlyUsingHardwareSetting() && orientationHasBeenChanged);

	if(settingHasBeenChanged)
	{
		SetHardwareOrientation(GetOrientationSetting());
		if(IsScreenOrientationChanged())
		{
			PrepareForScreenSizeChange();
		}
		if(checkOnTransition)
		{
			GetApp().SetMustSave();
		}
	}

	ApplyOrientationChange();

}


public void Reset (byte originalSetting) {

	SetOrientationSetting(originalSetting);
	DoCheckHandleOrientationChange(false);

}


public void ApplyOrientationChange () {

	 boolean settingIsAutomatic = GetOrientationSetting() == orientationAutomatic;


	SetCurrentlyUsedOrientation(GetOrientation());
	SetIsCurrentlyUsingHardwareSetting(settingIsAutomatic);

}


public void RefreshGlobalTimeRegistration () {

	 boolean needOnTime = false;

	
	needOnTime = needOnTime || (IsScreenOrientationChanged() && IsScreenOrientationChangeHardwareBased());



if(needOnTime != GetIsRegisteredInGlobalTime())
	{
		if(needOnTime)
		{
			RegisterInGlobalTime();
		}
		else
		{
			UnRegisterInGlobalTime();
		}
		SetIsRegisteredInGlobalTime(needOnTime);
	}

}


public void OnStartRenderingFrame () {


	
	if(IsScreenOrientationChanged() && IsScreenOrientationChangeHardwareBased() && !GetApp().IsInTransition())
	{
		DoTransitionAfterScreenOrientationChange();
	}

}


public void PrepareForScreenSizeChange () {


	GetApp().OnNeedToReloadGame();

}


public void DoTransitionAfterScreenOrientationChange () {



	
	
	
	
	 TetrisApp app = GetApp();
	 BaseScene scene = app.GetBaseScene();


	 int sceneTransitionCommandToRun = scene.GetExitSceneCommand();

	if(sceneTransitionCommandToRun == (((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct())) 
	{
		sceneTransitionCommandToRun = scene.GetCommandUsedToInstantiate();
	}

	 int commandToRun = scene.GetReloadSceneCommand();

	if(commandToRun != (((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct()))
	{
		scene.HandleCommand(commandToRun);
	}
	else
	{
		app.HandleSceneTransitionCommand(sceneTransitionCommandToRun);
	}

}


public boolean GetIsRegisteredInGlobalTime () {


	return  mIsRegisteredInGlobalTime;



}


public void SetIsRegisteredInGlobalTime (boolean b) {




mIsRegisteredInGlobalTime = b;

}


public byte GetCurrentlyUsedOrientation () {



return  mCurrentlyUsedOrientation;

}


public void SetCurrentlyUsedOrientation (byte ori) {



if(mCurrentlyUsedOrientation != ori)
	{
		mCurrentlyUsedOrientation = ori;
		RessourcesManager.Get().RefreshConstants();
	}

}


public boolean IsCurrentlyUsingHardwareSetting () {



return  mCurrentlyUsingHardwareSetting;

}


public void SetIsCurrentlyUsingHardwareSetting (boolean b) {



mCurrentlyUsingHardwareSetting = b;

}


public boolean[] mOrientationSupported = new boolean[orientationSettingCount];

public boolean mIsRegisteredInGlobalTime;
public byte mOrientationSetting;
public byte mCurrentlyUsedOrientation;
public boolean mCurrentlyUsingHardwareSetting;



}

