package ca.jamdat.tetris;

import ca.jamdat.flight.*;
import ca.jamdat.fuser.*;
import ca.jamdat.flight.String;
import ca.jamdat.flight.Package;


public class FileManager extends TimeControlled {

public final static short statusIdle = 0;
public final static short statusReading = 1;
public final static short statusWriting = 2;
public final static short statusDeleting = 3;
public final static short statusTerminated = 4;

public final static short subStatusUnknow = 0;
public final static short subStatusInit = 1;
public final static short subStatusLoading = 2;
public final static short subStatusReading = 3;
public final static short subStatusStartWriting = 4;
public final static short subStatusWriting = 5;
public final static short subStatusDeleting = 6;

public final static short opRead = 0;
public final static short opWrite = 1;

public  FileManager () {
mStatus = (statusIdle);
mSubStatus = (subStatusUnknow);




}


public void destruct() {

	


}


public void Terminate () {



	if(mFileLoader != null)
	{
		mFileLoader.Terminate();
		mFileLoader = null;
	}

	if(mGameFile != null)
	{
		mGameFile.Terminate();
		mGameFile = null;
	}




}


public void StartReadingProcess () {

	


	if(mStatus != statusReading)
	{	
		TaskStarted(statusReading);
	}

}


public void StartWritingProcess () {



	if(mStatus != statusWriting)
	{	
		TaskStarted(statusWriting);
	}

}


public void StartDeletingProcess () {



	if(mGameFile.GetStatus() != FileHandler.statusDeleted)
	{
		if(mStatus != statusDeleting)
		{	
			TaskStarted(statusDeleting);
		}
	}
	else
	{
		TaskFinished();
	}

}


public void OnTime (int totalTimeMs, int deltaTimeMs) {




	if(mSubStatus == subStatusInit)
	{
		OnSubInit();
	}

	if(mSubStatus == subStatusLoading)
	{
		OnSubLoading();
	}

	if(mSubStatus == subStatusReading)
	{
		OnSubReading();
	}

	if(mSubStatus == subStatusStartWriting)
	{
		OnSubStartWriting();
	}

	if(mSubStatus == subStatusWriting)
	{
		OnSubWriting();
	}






}


public FileHandler GetFile() {
 return  mGameFile; 
}


public boolean IsIdle() {
 return  mStatus == statusIdle; 
}


public void OnSubInit () {

	if(mGameFile == null || mGameFile.GetStatus() == FileHandler.statusDeleted)
	{
		InitializeFile();
		mSubStatus = subStatusLoading;
	}
	else
	{
		mSubStatus = GetStateAfterLoading();
	}

}


public void OnSubLoading () {

	mGameFile = mFileLoader.OnLoad();

	if(mGameFile != null)
	{
		mSubStatus = GetStateAfterLoading();
		mFileLoader = null;
	}

}


public void OnSubReading () {



	SerializeSegments(opRead);
	TaskFinished();

}


public void OnSubStartWriting () {



	 FileHandler gameFile = mGameFile;
	if(gameFile.GetStatus() != FileHandler.statusNotEnoughSpace)
	{

		SerializeSegments(opWrite);
		gameFile.StartWritingProcess();
		mSubStatus = subStatusWriting;
	}
	else
	{
		TaskFinished();
	}

}


public void OnSubWriting () {



	if(mGameFile.GetStatus() == FileHandler.statusReady)
	{
		TaskFinished();
	}

}


public void TaskStarted(short nextStatus) {
 mStatus = nextStatus; mSubStatus = subStatusInit; RegisterOnTime(); 
}


public void TaskFinished () {

	mStatus = statusIdle;
	mSubStatus = subStatusUnknow;

}


public void InitializeFile () {





	
	if(mGameFile != null)
	{
		mGameFile.Terminate();
		mGameFile = null;
	}


	
	
	
	 String basepath = GetApp().GetDataDir();
	 String filename = StringUtils.CreateString("app.bin");
	 String pathname = basepath.Add(filename);



int[] segmentSizes = {( (40) + (15) + (110) + (ScreenManager.orientationCount * Key.COUNT * InputMapping.kMappingElementCount * 4) + (50) + (400 + 0))};

	mFileLoader = new FileLoader();
	mFileLoader.Initialize(pathname, segmentSizes, 1, (int)((1<< 17) | (((01)) << 16) | (((01)) << 15) | (((01)) << 14) | (((01)) << 13) | (((00)) << 12) | (((00)) << 11) | (((00)) << 10) | (((00)) << 9) | (((00)) << 8) | (((00)) << 7) | (((00)) << 6) | (11)));

}


public void SerializeSegments (short op) {

	 FileHandler gameFile = mGameFile;
	 SerializableContainer fileContainer = GetApp().GetConfigFilesContainer();

	 boolean reading = op == opRead;

	 FileSegmentStream fileSegment = reading ? gameFile.GetInputSegmentStream(0) : gameFile.GetOutputSegmentStream(0);

	 boolean writing = !reading;


	if(reading && !fileSegment.HasValidData())
	{
		return ; 
	}

	for(int serializableIndex = 0;serializableIndex < SerializableContainer.kSerializableCount;serializableIndex ++ )
	{
		 Serializable obj = fileContainer.GetSerializableObject((int)serializableIndex);
		if(obj != null)
		{
			if(reading) obj.Read(fileSegment); else obj.Write(fileSegment);
		}
	}






{
		 ScreenManager obj = ScreenManager.Get();
		if(reading) obj.Read(fileSegment); else obj.Write(fileSegment);
	}
		
	{
		 LanguageManager obj = LanguageManager.Get();
		if(reading) obj.Read(fileSegment); else obj.Write(fileSegment);
	}

	if(writing)
	{
		
		fileSegment.SetValidDataFlag(true);
	}

}


public static TetrisApp GetApp () {
 
	return  TetrisApp.Get();

}


public short GetStateAfterLoading () {

	if(mStatus == statusReading)
	{
		return  subStatusReading;
	}
	else if(mStatus == statusWriting)
	{
		return  subStatusStartWriting;
	}
	else
	{
		return  subStatusDeleting;
	}

}


public void RegisterOnTime () {

	this.OnTime(0, 0);







}


public short mStatus;
public FileLoader mFileLoader;
public FileHandler mGameFile;
public short mSubStatus;

/* One dimension array */
public static FileManager[] InstArrayFileManager(int size) {
	FileManager[] newArray = new FileManager[size];
	for(int i = 0 ; i < size ; i++) newArray[i] = new FileManager();
	return newArray; }

/* Two dimension array */
public static FileManager[][] InstArrayFileManager(int size1, int size2) {
	FileManager[][] newArray = new FileManager[size1][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new FileManager[size2];
		for(int j = 0 ; j < size2 ; j++) newArray[i][j] = new FileManager(); }
	return newArray; }

/* Three dimension array */
public static FileManager[][][] InstArrayFileManager(int size1, int size2, int size3) {
	FileManager[][][] newArray = new FileManager[size1][][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new FileManager[size2][];
		for(int j = 0 ; j < size2 ; j++) {
			newArray[i][j] = new FileManager[size3];
			for(int k = 0 ; k < size3 ; k++) newArray[i][j][k] = new FileManager(); } }
	return newArray; }



}

