package ca.jamdat.tetris;

import ca.jamdat.flight.*;
import ca.jamdat.fuser.*;
import ca.jamdat.flight.String;
import ca.jamdat.flight.Package;


public class TetrisApp extends Application {

public final static byte appFile = 0;
public final static byte fileTypeCount = 1;

public final static byte kCreated = 0;
public final static byte kWaitingFirstOnDraw = 1;
public final static byte kInitialized = 2;

public  TetrisApp () {
mState = (kCreated);
mGameFlowState = (kGameFlowNormal);
mBlackBackgroundOnNextHourglass = (true);
mStringToolbox = new StringToolbox();
mInputControler = new TetrisInputControler();
mRessourcesManager = new RessourcesManager();
mLanguageManager = new LanguageManager();
mScreenManager = new ScreenManager();


	
	RegisterInGlobalTime();

	
	MapKey(Key.FIRE, Key.UNDEFINED);
	MapKey(Key.SOFT_SELECT, Key.UNDEFINED);
	


mConfigFilesContainer = new SerializableContainer();


SetLoadingSpriteVisible(true);



}


public void destruct() {



if(GetCurrentScene() != null)
	{
		GetCurrentScene().UnRegisterInGlobalTime();
	}

	UnRegisterInGlobalTime();

	SetLoadingSpriteVisible(false);

	if(mpBackgroundViewport != null)
	{
		mpBackgroundViewport.SetViewport(null);
	}

	if(mpTetrisMediaPlayer != null)
	{
		mpTetrisMediaPlayer.UnloadResources();
		mpTetrisMediaPlayer = null;
	}



RessourcesManager.Get().Destruct();




mInputControler.Destruct();

	mFileManager.Terminate();
	mFileManager = null;
	mConfigFilesContainer = null;



StringToolbox.Destruct();








}


public void Initialize () {

	


	
	
	InitializeBaseComponents();
OnReadFile();

	
	RessourcesManager.Get().GetPackage(PackageID.globalBackground);
	


FlMath.Seed(Application.GetRealTime());

	VibrationManager.Get().SetEnabled(true);
 LanguageManager lm = LanguageManager.Get();
	 short currentLang = lm.GetLanguage();


	
	
	
	 boolean useDefault = !LanguageManager.IsLanguageSupported(currentLang);

	
	if(useDefault)
	{
		lm.SetLanguage(LanguageManager.GetDefaultLanguage());
	}








{
		LaunchFirstScene();
	}


}


public void InitializeBaseComponents () {

	






mpTetrisMediaPlayer = new TetrisMediaPlayer();
	mFileManager = new FileManager();
	mInputControler.SetMapping(GetGameMapping());

	
	GetOptionSettings().ApplySettings();




}


public void InitializeLangDependentBaseComponents () {

	




}


public boolean OnMsg (Component source, int msg, int intParam) {

	







return  super.OnMsg(source, msg, intParam);

}


public static TetrisApp Get() {
 return  (TetrisApp)Application.GetInstance();
	
}


public void OnTime (int totalTimeMs, int deltaTimeMs) {

	












if(mState == kCreated)
	{
		mState = kWaitingFirstOnDraw;
	}
	else if(mState == kWaitingFirstOnDraw)
	{
		Initialize();
		mState = kInitialized;
	}
	
	if(mpBackgroundViewport == null && RessourcesManager.Get().IsPackageLoaded(PackageID.globalBackground))
	{


{
			LoadBackgroundResources();
		}
	}

	if(mMustSave)
	{
		Save();
		SetMustSave(false);
		
	}

	super.OnTime(totalTimeMs, deltaTimeMs);

}


public void OnKey (int key, boolean up) {

	mInputControler.OnKey(key, up);	

}


public void SetInputControlerEnabled (boolean enabled) {

	if(!enabled)
	{
		mInputControler.ReleaseAllKeys();
	}
	mInputControler.SetEnabled(enabled);

}


public void ReleaseAllKeys() {
 mInputControler.ReleaseAllKeys(); 
}


public void OnChargerEvent (boolean plugged) {




}


public void OnIncomingCall () {

	 BaseScene scene = TetrisApp.Get().GetBaseScene();
	if(scene != null)
	{
		scene.OnIncomingCall();
	}

}


public void OnFinishedCall () {



}


public void ArtificialInterruption () {

	Suspend();
	Resume();

}


public void Suspend () {

	



{
		 BaseScene currentScene = GetBaseScene();
		if(currentScene != null)
		{
			currentScene.Suspend();
		}
	}

}


public void Resume () {

	



{






}

}


public boolean IsInTransition() {
 return  super.IsInSceneTransition(); 
}


public BaseScene GetBaseScene () {

	

	return  (BaseScene)GetCurrentScene();

}


public boolean IsPausedWithFocus () {

	return  false;

}


public void OnPauseAppWithFocus () {









}


public void OnResumeAppWithFocus () {









}


public void SetNormalInterruptedState() {
 mGameFlowState = kGameFlowNormal; 
}


public ScreenManager GetScreenManager () {

	return  mScreenManager;

}


public LanguageManager GetLanguageManager () {

	return  mLanguageManager;

}


public RessourcesManager GetRessourcesManager () {

	return  mRessourcesManager;

}


public TetrisState GetTetrisState () {
 
	return ((TetrisState)mConfigFilesContainer.GetSerializableObject(SerializableContainer.kTetrisStateIndex));

}


public TetrisMediaPlayer GetPlayerPtr() {
 return  mpTetrisMediaPlayer; 
}


public TetrisMediaPlayer GetPlayer() {
 ; return GetPlayerPtr(); 
}


public TetrisInputMapping GetGameMapping () {

	return ((TetrisInputMapping)mConfigFilesContainer.GetSerializableObject(SerializableContainer.kTetrisInputMappingIndex));

}


public void SetMustSave (boolean state) {

	mMustSave = state;

}


public void LaunchFirstScene () {

	

	 int nextSceneCmd = 0;


	
	InitializeLangDependentBaseComponents();











{
			nextSceneCmd = (((int)ConstantsEnglishPortrait.goToSplashSceneCommand == (int)ConstantsEnglishLandscape.goToSplashSceneCommand) ? Constants.goToSplashSceneCommand : ConstantsGetter.GetConstants().goToSplashSceneCommandFct());
		}
	HandleSceneTransitionCommand(nextSceneCmd);

}


public void PrepareToLaunchGameScene () {

	

	
	GetPlayer().StopSound();

	UnstackAllScenes();
	RessourcesManager.Get().RemovePackageFromCache(PackageID.commonMenu);


}


public boolean IsBackCommand (int command) {

	return 	   command == (((int)ConstantsEnglishPortrait.backCommand == (int)ConstantsEnglishLandscape.backCommand) ? Constants.backCommand : -123) || command == (((int)ConstantsEnglishPortrait.backFromOptionsMenuCommand == (int)ConstantsEnglishLandscape.backFromOptionsMenuCommand) ? Constants.backFromOptionsMenuCommand : -123) || command == (((int)ConstantsEnglishPortrait.resetStatsCommand == (int)ConstantsEnglishLandscape.resetStatsCommand) ? Constants.resetStatsCommand : -123) || command == (((int)ConstantsEnglishPortrait.selectDisplayOrientationCommand == (int)ConstantsEnglishLandscape.selectDisplayOrientationCommand) ? Constants.selectDisplayOrientationCommand : -123);

}


public void HandleSceneTransitionCommand (int command) {

	



	if(IsInSceneTransition()) 
	{
		return ;
	}











 int exitingScenePackageID = PackageID.invalid;

	if(GetBaseScene() != null)
	{
		exitingScenePackageID = GetBaseScene().GetPackageID();
	}

	 boolean exitingFromRootScene = exitingScenePackageID == PackageID.mainMenuScene;


	TakeFocus();

	 boolean needToReloadPackages = false;

	 LanguageManager lm = LanguageManager.Get();
	if(lm.IslanguageChanged())
	{
		needToReloadPackages = true;
		lm.ApplyLanguageChange();
	}
	 ScreenManager sm = ScreenManager.Get();
	 boolean hadHardwareResolutionChange = false;


	if(sm.IsScreenOrientationChanged())
	{
	

hadHardwareResolutionChange = sm.IsScreenOrientationChangeHardwareBased();
		needToReloadPackages = true;
	}

	sm.CheckHandleOrientationChange();

	 boolean isBackCommand = IsBackCommand(command);


	if((exitingFromRootScene && hadHardwareResolutionChange))
	{
		AskClearStackToMainMenuAtNextSceneTransition();
	}

	 boolean neverSeeThisSceneAgain = needToReloadPackages && ((isBackCommand && !exitingFromRootScene) || hadHardwareResolutionChange);


	if(neverSeeThisSceneAgain)
	{

		GetBaseScene().SetSkipOnBack(true);


}

	 Scene sceneToLaunch = null;

	if(needToReloadPackages)
	{
	
		if(hadHardwareResolutionChange)
		{
			mHardwareOrientationChanging = true;
		}
	 BaseScene newScene = null;

		newScene = new BaseScene(PackageID.emptyScene);
		newScene.SetExitSceneCommand(command);


sceneToLaunch = newScene;
		command = (((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct());
	}


	if(exitingScenePackageID != PackageID.emptyScene)
{
	
		if(!needToReloadPackages)
		{
			mHardwareOrientationChanging = false;
		}
	}

	 boolean stackHasBeenCleared = mClearSceneToMainMenuAtNextTransition;

	if(stackHasBeenCleared)
	{
		if(isBackCommand)
		{
			UnstackAllScenes();
			sceneToLaunch = new SelectorScene(PackageID.mainMenuScene);
			mClearSceneToMainMenuAtNextTransition = false;
		}
		else
		{
			ClearStackToMainMenu();
		}
	}

	 boolean dismissCurrentScene = isBackCommand && !needToReloadPackages;

	if(dismissCurrentScene)
	{
		if(!stackHasBeenCleared)
		{
			Scene.Dismiss(0);
		}
	}
	else if(sceneToLaunch == null)
	{
		sceneToLaunch = InstanciateSceneFromCommand(command);
	}
	
	if(sceneToLaunch != null)
	{

		((BaseScene)sceneToLaunch).SetCommandUsedToInstantiate(command);
RessourcesManager.Get().RefreshBackgroundPackage(GetScenePackageIDBasedOnCommand(sceneToLaunch, command));

		sceneToLaunch.SetViewport(this);

		Scene.Launch(sceneToLaunch);







}

}


public boolean MustShowTutorialBeforeCurrentGameMode () {



 TetrisState tetrisState = GetTetrisState();
	 byte tutShowing = tetrisState.GetTutorialShowingSetting();

	return	(tutShowing == TetrisState.kTutorialShowingAlways ||
			(tutShowing == TetrisState.kTutorialShowingOnce &&
			!tetrisState.IsGameModeTutorialShown(tetrisState.GetGameMode())));

}


public Scene InstanciateSceneFromCommand (int command) {

	 Scene sceneToLaunch = null;

 	 TetrisState tetrisState = GetTetrisState();


switch(command)
	{
	case (((int)ConstantsEnglishPortrait.goToGameSceneCommand == (int)ConstantsEnglishLandscape.goToGameSceneCommand) ? Constants.goToGameSceneCommand : -123) :
		{
			
			SetNormalInterruptedState();

			PrepareToLaunchGameScene();
			sceneToLaunch = new GameScene(PackageID.gameScene);
		}
		break;


	case (((int)ConstantsEnglishPortrait.goToTutorialMenuCommand == (int)ConstantsEnglishLandscape.goToTutorialMenuCommand) ? Constants.goToTutorialMenuCommand : -123) :
		{
			sceneToLaunch = new SelectorScene(PackageID.tutorialMenuScene);
		}
		break;

	case (((int)ConstantsEnglishPortrait.startGameFromTutorialCommand == (int)ConstantsEnglishLandscape.startGameFromTutorialCommand) ? Constants.startGameFromTutorialCommand : -123) :
		{
			tetrisState.ShownGameModeTutorial(tetrisState.GetGameMode());
			tetrisState.StartGame();
			sceneToLaunch = InstanciateSceneFromCommand((((int)ConstantsEnglishPortrait.goToGameSceneCommand == (int)ConstantsEnglishLandscape.goToGameSceneCommand) ? Constants.goToGameSceneCommand : ConstantsGetter.GetConstants().goToGameSceneCommandFct()));
		}
		break;
case (((int)ConstantsEnglishPortrait.goToPauseSceneCommand == (int)ConstantsEnglishLandscape.goToPauseSceneCommand) ? Constants.goToPauseSceneCommand : -123) :
		{
			sceneToLaunch = new SelectorScene(PackageID.pauseMenuScene);
		}
		break;















case (((int)ConstantsEnglishPortrait.goToMainMenuCommand == (int)ConstantsEnglishLandscape.goToMainMenuCommand) ? Constants.goToMainMenuCommand : -123) :
		{
			
			tetrisState.DeleteGameState();

			UnstackAllScenes();

			sceneToLaunch = new SelectorScene(PackageID.mainMenuScene);




}
		break;


	case (((int)ConstantsEnglishPortrait.goToStatisticsMenuCommand == (int)ConstantsEnglishLandscape.goToStatisticsMenuCommand) ? Constants.goToStatisticsMenuCommand : -123) :
		{
			sceneToLaunch = new BaseScene(PackageID.statisticsMenuScene);
		}
		break;
case (((int)ConstantsEnglishPortrait.goToGameOverMenuCommand == (int)ConstantsEnglishLandscape.goToGameOverMenuCommand) ? Constants.goToGameOverMenuCommand : -123) :
		{
			




sceneToLaunch = new ResultsScene(GetGameOverPackage());
}
		break;
























case (((int)ConstantsEnglishPortrait.goToOptionsMenuCommand == (int)ConstantsEnglishLandscape.goToOptionsMenuCommand) ? Constants.goToOptionsMenuCommand : -123) :
		{
			sceneToLaunch = new OptionsScene(PackageID.optionsMenuScene, false);
		}
		break;

	case (((int)ConstantsEnglishPortrait.goToOptionsMenuFromPauseCommand == (int)ConstantsEnglishLandscape.goToOptionsMenuFromPauseCommand) ? Constants.goToOptionsMenuFromPauseCommand : -123) :
		{
			sceneToLaunch = new OptionsScene(PackageID.optionsMenuScene, true);
		}
		break;


	case (((int)ConstantsEnglishPortrait.selectSinglePlayerMenuCommand == (int)ConstantsEnglishLandscape.selectSinglePlayerMenuCommand) ? Constants.selectSinglePlayerMenuCommand : -123) :
		{
			tetrisState.SetGameMode(false);
			sceneToLaunch = InstanciateSceneFromCommand((((int)ConstantsEnglishPortrait.goToDelayLoadingScreenCommand == (int)ConstantsEnglishLandscape.goToDelayLoadingScreenCommand) ? Constants.goToDelayLoadingScreenCommand : ConstantsGetter.GetConstants().goToDelayLoadingScreenCommandFct()));
			mGameFlowState = kGameFlowSinglePlayerLoading;
		}
		break;

	case (((int)ConstantsEnglishPortrait.goToDelayLoadingScreenCommand == (int)ConstantsEnglishLandscape.goToDelayLoadingScreenCommand) ? Constants.goToDelayLoadingScreenCommand : -123) :
		{
			sceneToLaunch = new DelayLoadingScene(PackageID.delayLoadingMenuScene);
		}
		break;


	case (((int)ConstantsEnglishPortrait.goToResetStatsPromptCommand == (int)ConstantsEnglishLandscape.goToResetStatsPromptCommand) ? Constants.goToResetStatsPromptCommand : -123) :
		{
			sceneToLaunch = new TetrisPromptScene(PackageID.promptMenuScene, RSRC_promptMenuScenePackage.optionsResetStatsTitleString_ID, RSRC_promptMenuScenePackage.optionsResetStatsPromptString_ID, RSRC_promptMenuScenePackage.selectSoftKeyString_ID, (((int)ConstantsEnglishPortrait.chooseSelectedItemCommand == (int)ConstantsEnglishLandscape.chooseSelectedItemCommand) ? Constants.chooseSelectedItemCommand : ConstantsGetter.GetConstants().chooseSelectedItemCommandFct()), RSRC_promptMenuScenePackage.backSoftKeyString_ID, (((int)ConstantsEnglishPortrait.backCommand == (int)ConstantsEnglishLandscape.backCommand) ? Constants.backCommand : ConstantsGetter.GetConstants().backCommandFct()), true, (((int)ConstantsEnglishPortrait.backCommand == (int)ConstantsEnglishLandscape.backCommand) ? Constants.backCommand : ConstantsGetter.GetConstants().backCommandFct()), (((int)ConstantsEnglishPortrait.resetStatsCommand == (int)ConstantsEnglishLandscape.resetStatsCommand) ? Constants.resetStatsCommand : ConstantsGetter.GetConstants().resetStatsCommandFct()), (((int)ConstantsEnglishPortrait.promptMenuNoIndex == (int)ConstantsEnglishLandscape.promptMenuNoIndex) ? Constants.promptMenuNoIndex : ConstantsGetter.GetConstants().promptMenuNoIndexFct()));
		}
		break;
case (((int)ConstantsEnglishPortrait.goToGameSetupMenuSinglePlayerCommand == (int)ConstantsEnglishLandscape.goToGameSetupMenuSinglePlayerCommand) ? Constants.goToGameSetupMenuSinglePlayerCommand : -123) :
	case (((int)ConstantsEnglishPortrait.goToSingleGameSettingsCommand == (int)ConstantsEnglishLandscape.goToSingleGameSettingsCommand) ? Constants.goToSingleGameSettingsCommand : -123) :
		{
			sceneToLaunch = new SelectorScene(PackageID.singlePlayerModesMenuScene);
		}
		break;
	

	case (((int)ConstantsEnglishPortrait.goToDisplayOrientationMenuCommand == (int)ConstantsEnglishLandscape.goToDisplayOrientationMenuCommand) ? Constants.goToDisplayOrientationMenuCommand : -123) :
		{
			sceneToLaunch = new SelectorScene(PackageID.displayOrientationMenuScene);
		}
		break;		
case (((int)ConstantsEnglishPortrait.goToGameSetupCommand == (int)ConstantsEnglishLandscape.goToGameSetupCommand) ? Constants.goToGameSetupCommand : -123) :
		{
			sceneToLaunch = new GameSetupScene(PackageID.gameSetupMenuScene);
		}
		break;








case (((int)ConstantsEnglishPortrait.startGameCommand == (int)ConstantsEnglishLandscape.startGameCommand) ? Constants.startGameCommand : -123) :
		{
			tetrisState.CreateGame(false);

			if(MustShowTutorialBeforeCurrentGameMode())
			{
				sceneToLaunch = InstanciateSceneFromCommand((((int)ConstantsEnglishPortrait.goToTutorialMenuCommand == (int)ConstantsEnglishLandscape.goToTutorialMenuCommand) ? Constants.goToTutorialMenuCommand : ConstantsGetter.GetConstants().goToTutorialMenuCommandFct()));
			}
			else
			{
				tetrisState.StartGame();
				
				sceneToLaunch = InstanciateSceneFromCommand((((int)ConstantsEnglishPortrait.goToGameSceneCommand == (int)ConstantsEnglishLandscape.goToGameSceneCommand) ? Constants.goToGameSceneCommand : ConstantsGetter.GetConstants().goToGameSceneCommandFct()));
			}
		}
		break;

	case (((int)ConstantsEnglishPortrait.playAgainCommand == (int)ConstantsEnglishLandscape.playAgainCommand) ? Constants.playAgainCommand : -123) :
		{
			tetrisState.RestartGame();

			PrepareToLaunchGameScene();
			sceneToLaunch = new GameScene(PackageID.gameScene);
		}
		break;

	case (((int)ConstantsEnglishPortrait.goToAboutMenuCommand == (int)ConstantsEnglishLandscape.goToAboutMenuCommand) ? Constants.goToAboutMenuCommand : -123) :
		{
			sceneToLaunch = new BaseScene(PackageID.aboutMenuScene);
		}
		break;







case (((int)ConstantsEnglishPortrait.goToSplashSceneCommand == (int)ConstantsEnglishLandscape.goToSplashSceneCommand) ? Constants.goToSplashSceneCommand : -123) :
		{
			sceneToLaunch = new SplashScene(PackageID.splashMenuScene);
		}
		break;






case (((int)ConstantsEnglishPortrait.goToHelpMenuCommand == (int)ConstantsEnglishLandscape.goToHelpMenuCommand) ? Constants.goToHelpMenuCommand : -123) :
		{
			sceneToLaunch = new SelectorScene(PackageID.helpMenuScene);
		}
		break;

	case (((int)ConstantsEnglishPortrait.goToReturnToMainPromptCommand == (int)ConstantsEnglishLandscape.goToReturnToMainPromptCommand) ? Constants.goToReturnToMainPromptCommand : -123) :
		{
			sceneToLaunch = new TetrisPromptScene(PackageID.promptMenuScene, RSRC_promptMenuScenePackage.returnMainMenuTitleString_ID, RSRC_promptMenuScenePackage.returnMainMenuPromptString_ID, RSRC_promptMenuScenePackage.selectSoftKeyString_ID, (((int)ConstantsEnglishPortrait.chooseSelectedItemCommand == (int)ConstantsEnglishLandscape.chooseSelectedItemCommand) ? Constants.chooseSelectedItemCommand : ConstantsGetter.GetConstants().chooseSelectedItemCommandFct()), RSRC_promptMenuScenePackage.backSoftKeyString_ID, (((int)ConstantsEnglishPortrait.backCommand == (int)ConstantsEnglishLandscape.backCommand) ? Constants.backCommand : ConstantsGetter.GetConstants().backCommandFct()), true, (((int)ConstantsEnglishPortrait.backCommand == (int)ConstantsEnglishLandscape.backCommand) ? Constants.backCommand : ConstantsGetter.GetConstants().backCommandFct()), (((int)ConstantsEnglishPortrait.goToMainMenuCommand == (int)ConstantsEnglishLandscape.goToMainMenuCommand) ? Constants.goToMainMenuCommand : ConstantsGetter.GetConstants().goToMainMenuCommandFct()), (((int)ConstantsEnglishPortrait.promptMenuYesIndex == (int)ConstantsEnglishLandscape.promptMenuYesIndex) ? Constants.promptMenuYesIndex : ConstantsGetter.GetConstants().promptMenuYesIndexFct()));
		}
		break;



















case (((int)ConstantsEnglishPortrait.goToPlayAgainPromptCommand == (int)ConstantsEnglishLandscape.goToPlayAgainPromptCommand) ? Constants.goToPlayAgainPromptCommand : -123) :
		{
			sceneToLaunch = new TetrisPromptScene(PackageID.promptMenuScene, RSRC_promptMenuScenePackage.againTitleString_ID, RSRC_promptMenuScenePackage.againPromptString_ID, RSRC_promptMenuScenePackage.selectSoftKeyString_ID, (((int)ConstantsEnglishPortrait.chooseSelectedItemCommand == (int)ConstantsEnglishLandscape.chooseSelectedItemCommand) ? Constants.chooseSelectedItemCommand : ConstantsGetter.GetConstants().chooseSelectedItemCommandFct()), RSRC_promptMenuScenePackage.emptyString_ID, (((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct()), true, (((int)ConstantsEnglishPortrait.goToMainMenuCommand == (int)ConstantsEnglishLandscape.goToMainMenuCommand) ? Constants.goToMainMenuCommand : ConstantsGetter.GetConstants().goToMainMenuCommandFct()), (((int)ConstantsEnglishPortrait.playAgainCommand == (int)ConstantsEnglishLandscape.playAgainCommand) ? Constants.playAgainCommand : ConstantsGetter.GetConstants().playAgainCommandFct()), (((int)ConstantsEnglishPortrait.promptMenuYesIndex == (int)ConstantsEnglishLandscape.promptMenuYesIndex) ? Constants.promptMenuYesIndex : ConstantsGetter.GetConstants().promptMenuYesIndexFct()));
		}
		break;



































































case (((int)ConstantsEnglishPortrait.goToExitPromptCommand == (int)ConstantsEnglishLandscape.goToExitPromptCommand) ? Constants.goToExitPromptCommand : -123) :
	case (((int)ConstantsEnglishPortrait.goToExitPromptFromPauseCommand == (int)ConstantsEnglishLandscape.goToExitPromptFromPauseCommand) ? Constants.goToExitPromptFromPauseCommand : -123) :
		{
			sceneToLaunch = new TetrisPromptScene(PackageID.promptMenuScene, RSRC_promptMenuScenePackage.exitGameTitleString_ID, command == (((int)ConstantsEnglishPortrait.goToExitPromptCommand == (int)ConstantsEnglishLandscape.goToExitPromptCommand) ? Constants.goToExitPromptCommand : ConstantsGetter.GetConstants().goToExitPromptCommandFct()) ? RSRC_promptMenuScenePackage.exitGamePromptString_ID : RSRC_promptMenuScenePackage.exitGameFromPausePromptString_ID, RSRC_promptMenuScenePackage.selectSoftKeyString_ID, (((int)ConstantsEnglishPortrait.chooseSelectedItemCommand == (int)ConstantsEnglishLandscape.chooseSelectedItemCommand) ? Constants.chooseSelectedItemCommand : ConstantsGetter.GetConstants().chooseSelectedItemCommandFct()), RSRC_promptMenuScenePackage.backSoftKeyString_ID, (((int)ConstantsEnglishPortrait.backCommand == (int)ConstantsEnglishLandscape.backCommand) ? Constants.backCommand : ConstantsGetter.GetConstants().backCommandFct()), true, (((int)ConstantsEnglishPortrait.backCommand == (int)ConstantsEnglishLandscape.backCommand) ? Constants.backCommand : ConstantsGetter.GetConstants().backCommandFct()), (((int)ConstantsEnglishPortrait.exitGameCommand == (int)ConstantsEnglishLandscape.exitGameCommand) ? Constants.exitGameCommand : ConstantsGetter.GetConstants().exitGameCommandFct()), (((int)ConstantsEnglishPortrait.promptMenuYesIndex == (int)ConstantsEnglishLandscape.promptMenuYesIndex) ? Constants.promptMenuYesIndex : ConstantsGetter.GetConstants().promptMenuYesIndexFct()));
		}
		break;

	case (((int)ConstantsEnglishPortrait.goToFreeStorageSpacePromptCommand == (int)ConstantsEnglishLandscape.goToFreeStorageSpacePromptCommand) ? Constants.goToFreeStorageSpacePromptCommand : -123) :
		{
			sceneToLaunch = new TetrisPromptScene(PackageID.promptMenuScene, RSRC_promptMenuScenePackage.freeStorageSpaceTitleString_ID, RSRC_promptMenuScenePackage.freeStorageSpacePromptString_ID, RSRC_promptMenuScenePackage.emptyString_ID, (((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct()), RSRC_promptMenuScenePackage.exitSoftKeyString_ID, (((int)ConstantsEnglishPortrait.exitGameCommand == (int)ConstantsEnglishLandscape.exitGameCommand) ? Constants.exitGameCommand : ConstantsGetter.GetConstants().exitGameCommandFct()), false, (((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct()), (((int)ConstantsEnglishPortrait.noneCommand == (int)ConstantsEnglishLandscape.noneCommand) ? Constants.noneCommand : ConstantsGetter.GetConstants().noneCommandFct()), (((int)ConstantsEnglishPortrait.promptMenuNoIndex == (int)ConstantsEnglishLandscape.promptMenuNoIndex) ? Constants.promptMenuNoIndex : ConstantsGetter.GetConstants().promptMenuNoIndexFct()));
		}
		break;




















case (((int)ConstantsEnglishPortrait.goToOptionsMenuFromGameSetupCommand == (int)ConstantsEnglishLandscape.goToOptionsMenuFromGameSetupCommand) ? Constants.goToOptionsMenuFromGameSetupCommand : -123) :
		{
			ClearStackToMainMenu();
			sceneToLaunch = InstanciateSceneFromCommand((((int)ConstantsEnglishPortrait.goToOptionsMenuCommand == (int)ConstantsEnglishLandscape.goToOptionsMenuCommand) ? Constants.goToOptionsMenuCommand : ConstantsGetter.GetConstants().goToOptionsMenuCommandFct()));
		}
		break;






default :
		{

		}
		break;
	};



	return  sceneToLaunch;

}


public void ClearStackToMainMenu () {

	

	BaseScene sceneToLaunch = null;

	UnstackAllScenes();
	sceneToLaunch = new SelectorScene((byte)-PackageID.mainMenuScene);	
	sceneToLaunch.SetViewport(this);
	Scene.Launch(sceneToLaunch);

	mClearSceneToMainMenuAtNextTransition = false;

}


public void ClearStackToScene (byte sceneToStopPackageID) {

	



 BaseScene currentScene = ((BaseScene)GetCurrentScene());
	while(currentScene != null && currentScene.GetPackageID() != sceneToStopPackageID)
	{
		GetCurrentScene().Dismiss((((int)ConstantsEnglishPortrait.automaticDismissCommand == (int)ConstantsEnglishLandscape.automaticDismissCommand) ? Constants.automaticDismissCommand : ConstantsGetter.GetConstants().automaticDismissCommandFct()));
		currentScene = ((BaseScene)GetCurrentScene());
	}



}


public void AskClearStackToMainMenuAtNextSceneTransition() {
 mClearSceneToMainMenuAtNextTransition = true; 
}


public static boolean IsGameOverPackage (byte packageID) {

	

		return (packageID == PackageID.marathonScoreMenuScene ||
			packageID == PackageID.ultraScoreMenuScene ||
			packageID == PackageID.fortyLinesScoreMenuScene);

}


public byte GetGameOverPackage () {

	

	 byte packageID = PackageID.invalid;


	switch(GetTetrisState().GetGameMode())
	{
		
		case TetrisState.kModeMarathon :
			packageID = PackageID.marathonScoreMenuScene;
			break;
		case TetrisState.kModeUltra :
			packageID = PackageID.ultraScoreMenuScene;
			break;
		case TetrisState.kMode40Lines :
			packageID = PackageID.fortyLinesScoreMenuScene;
			break;





default : ;
	}

	return  packageID;

}


public boolean IsHardwareOrientationChanging () {


	return  mHardwareOrientationChanging;



}


public byte GetScenePackageIDBasedOnCommand (Scene scene, int command) {



{
		return  ((BaseScene)scene).GetPackageID();
	}

}


public byte GetInGameMusicPackageID () {


	 byte mode = GetOptionSettings().GetMusicMode();

	if(mode == InGameMusicMode.kInGameMusic1)
	{
		return  PackageID.inGameMusic1;
	}
	return  PackageID.inGameMusic2;



}


public void UnloadMenuDependencies () {

	

	RessourcesManager.Get().RemovePackageFromCache(PackageID.commonMenu);


	RessourcesManager.Get().RemovePackageFromCache(PackageID.menuStrings);

}


public void SetLoadingSpriteVisible (boolean visible) {

	



 Package p = RessourcesManager.Get().GetPackage(PackageID.loadingHourGlass, false);

	 Viewport viewport = null;
	(viewport)= (viewport).Cast((p).GetEntryPoint(RSRC_loadingHourGlassPackage.loadingImageViewport_ID), null);

	 Shape pFirstFrameBackgroundShape = null;
	(pFirstFrameBackgroundShape)= (pFirstFrameBackgroundShape).Cast((p).GetEntryPoint(RSRC_loadingHourGlassPackage.firstFrameBackgroundShape_ID), null);


	if(visible)
	{

		
		
		
		Component c = null;

		(c)= (c).Cast((p).GetEntryPoint(RSRC_loadingHourGlassPackage.loadingSprite_ID), null);
		c.SetTopLeft((short)(((int)ConstantsEnglishPortrait.loadingSpriteLeft == (int)ConstantsEnglishLandscape.loadingSpriteLeft) ? Constants.loadingSpriteLeft : ConstantsGetter.GetConstants().loadingSpriteLeftFct()), (short)(((int)ConstantsEnglishPortrait.loadingSpriteTop == (int)ConstantsEnglishLandscape.loadingSpriteTop) ? Constants.loadingSpriteTop : ConstantsGetter.GetConstants().loadingSpriteTopFct()));

		pFirstFrameBackgroundShape.SetRect((short)0, (short)0, (short)(((int)ConstantsEnglishPortrait.screenWidth == (int)ConstantsEnglishLandscape.screenWidth) ? Constants.screenWidth : ConstantsGetter.GetConstants().screenWidthFct()), (short)(((int)ConstantsEnglishPortrait.screenHeight == (int)ConstantsEnglishLandscape.screenHeight) ? Constants.screenHeight : ConstantsGetter.GetConstants().screenHeightFct()));
if(mBlackBackgroundOnNextHourglass)
		{
			pFirstFrameBackgroundShape.SetViewport(this);

			mBlackBackgroundOnNextHourglass = false;
		}

		viewport.SetViewport(this);

	}
	else
	{
		viewport.SetViewport(null);
		pFirstFrameBackgroundShape.SetViewport(null);
	}

}


public void SetBackgroundVisible (boolean visible) {

	if(mpBackgroundViewport != null)
	{
		mpBackgroundViewport.SetVisible(visible);
	}







}


public void SetSoftKeySpriteVisible (boolean visible) {

	

	
	if(mpBottomSoftKeyBackgroundSprite != null)
	{
		 byte ori = ScreenManager.Get().GetOrientation();

		 boolean bottomVisible = ori != ScreenManager.orientationFlip && visible;

		 boolean topVisible = ori != ScreenManager.orientationStraight && visible;

			
		mpBottomSoftKeyBackgroundSprite.SetVisible(bottomVisible);



}

}


public void SetSoftKeySpriteVisible (boolean visible, boolean atTop, boolean atLeft) {

























}


public static void ReplaceComponent (Component dest, Component source) {

	

	 Component previous = dest.GetPreviousSiblingComponent();
	source.SetViewport(dest.GetViewport());
	dest.SetViewport(null);
	source.SetTopLeft(dest.GetRectLeft(), dest.GetRectTop());
	dest.SetTopLeft((short)0, (short)0);
	if(previous != null)
	{
		source.PutInFront(previous);
	}

}


public static void UniversalSleep (int sleepTimeMs) {

	

















}


public static void StartAndInitializeTimeSystem (TimeSystem pTimeSystem, int initialTime) {

	

	pTimeSystem.Start();
	pTimeSystem.SetTotalTime(initialTime);
	pTimeSystem.OnTime(0, 0);

}


public static void FinalizeAndStopTimeSystem (TimeSystem pTimeSystem, int finalTime) {

	

	pTimeSystem.SetTotalTime(finalTime);
	pTimeSystem.OnTime(0, 0);
	pTimeSystem.Stop();

}


public static TimeSystem GetAndRegisterAndStopTimeSystem (Package pPackage, int index) {

	

	 TimeSystem pTimeSystem = null;
	(pTimeSystem)= (pTimeSystem).Cast((pPackage).GetEntryPoint(index), null);
	pTimeSystem.RegisterInGlobalTime();
	pTimeSystem.OnTime(0, 0);
	pTimeSystem.Stop();
	return  pTimeSystem;

}


public static boolean IsInClippingRect (DisplayContext displayContext, short destRect_left, short destRect_top, short destRect_width, short destRect_height) {

	

	
	 short clipRect_left = displayContext.GetClippingRectLeft();
  short clipRect_top = displayContext.GetClippingRectTop();
  short clipRect_width = displayContext.GetClippingRectWidth();
  short clipRect_height = displayContext.GetClippingRectHeight();

	{  int clipperRight = (clipRect_left + clipRect_width);
  int clippedRight = (destRect_left + destRect_width);
  int clipperBottom = (clipRect_top + clipRect_height);
  int clippedBottom = (destRect_top + destRect_height);
 if(clipRect_left > destRect_left) destRect_left = clipRect_left; if(clipRect_top > destRect_top) destRect_top = clipRect_top; if(clipperRight < clippedRight) clippedRight = clipperRight; if(clipperBottom < clippedBottom) clippedBottom = clipperBottom; destRect_width = (short)(clippedRight - destRect_left); destRect_height = (short)(clippedBottom - destRect_top); };
	return  !(destRect_width <= 0|| destRect_height <= 0);

}


public static void GenerateRandomString (String toScramble, String previous, int seed, Font f, int maxWidth) {

	

	
	
	
	
	
	
	
	
	
	{
		 int lenPrevious = f.GetLineWidth(previous);

		if(lenPrevious > maxWidth)
		{
			maxWidth = lenPrevious;
		}
	}

	 int bigPrimeNumber = 997;


	 Package p = RessourcesManager.Get().GetPackage(PackageID.common, false);
	 String scrambleRef = null;
	(scrambleRef)= (scrambleRef).Cast((p).GetEntryPoint(RSRC_commonPackage.scrambleReferenceString_ID), null);



	 int length = toScramble.GetLength();


	 byte currentChar = (byte)'a';

	 int indexScramble = 0;

	int charIndex;


	for(charIndex = 0;charIndex < length;charIndex ++ )
	{
		currentChar = toScramble.GetCharAt(charIndex);
		if(currentChar != (byte)' ')
		{
			indexScramble = scrambleRef.FindChar(currentChar);
			indexScramble += seed;
			indexScramble = indexScramble & 31;
			toScramble.ReplaceCharAt(charIndex, scrambleRef.GetCharAt(indexScramble));
		}
	}

	charIndex = seed;


	if(f.GetLineWidth(toScramble) > maxWidth)
	{
		
		
		
		
		do
		{
			charIndex = (charIndex + bigPrimeNumber) % length;
			toScramble.ReplaceCharAt(charIndex, previous.GetCharAt(charIndex));
		}
		while(f.GetLineWidth(toScramble) > maxWidth);

		int charIndex2;

		byte char1;

		byte char2;

		
		
		for(int i = 0;i < length;i ++)
		{
			charIndex = (charIndex + bigPrimeNumber) % length;
			charIndex2 = (charIndex + seed) % length;
			char1 = toScramble.GetCharAt(charIndex);
			char2 = toScramble.GetCharAt(charIndex2);
			if(char1 == ' ' || char2 == ' ')
			{
				continue;
			}

			toScramble.ReplaceCharAt(charIndex, char2);
			toScramble.ReplaceCharAt(charIndex2, char1);
		}
	}

}


public Font GetScramblingFont () {

	

	 Package p = RessourcesManager.Get().GetPackage(PackageID.font, false);
	 Font scrambleFont = null;
	(scrambleFont)= (scrambleFont).Cast((p).GetEntryPoint(RSRC_fontPackage.menuItemSelectedFont_ID), null);
	return  scrambleFont;

}


public Viewport GetBackgroundViewport() {
 return  mpBackgroundViewport; 
}


public void SetBackgroundViewport(Viewport v) {
 mpBackgroundViewport = v; 
}


public boolean IsInGameLoadingFlow() {
 return  mGameFlowState != kGameFlowNormal; 
}


public void OnNeedToReloadGame () {

	 TetrisMediaPlayer mediaPlayer = GetPlayerPtr();
	if(mediaPlayer != null)
	{
		mediaPlayer.StopSound();
	}

	 BaseScene scene = GetBaseScene();
	if(scene != null)
	{
		scene.OnNeedToReloadGame();
	}

	HideGlobalRessourcesShowHourglass();

}


public void HideGlobalRessourcesShowHourglass () {



	 boolean sizeIsValid = false;

	
	sizeIsValid = mpBackgroundViewport != null &&
				  mpBackgroundViewport.GetRectWidth() == (((int)ConstantsEnglishPortrait.screenWidth == (int)ConstantsEnglishLandscape.screenWidth) ? Constants.screenWidth : ConstantsGetter.GetConstants().screenWidthFct()) &&
				  mpBackgroundViewport.GetRectHeight() == (((int)ConstantsEnglishPortrait.screenHeight == (int)ConstantsEnglishLandscape.screenHeight) ? Constants.screenHeight : ConstantsGetter.GetConstants().screenHeightFct());

	mBlackBackgroundOnNextHourglass = !sizeIsValid;
	SetBackgroundVisible(sizeIsValid);

	 BaseScene scene = GetBaseScene();
	if(scene != null && scene.IsLoaded() && scene.GetPackageID() != PackageID.emptyScene)
	{
		scene.SetVisible(sizeIsValid);
	}

}


public boolean IsExitOnMoreGames () {

	



return  true;

}


public void StartUpLicenseCheckCompleted () {







}


public SerializableContainer GetConfigFilesContainer() {
 return  mConfigFilesContainer; 
}


public OptionSettings GetOptionSettings () {

	return  (OptionSettings)mConfigFilesContainer.GetSerializableObject(SerializableContainer.kOptionSettingsIndex);

}


public void SwitchToNGageApp () {




}


public byte GetGameModeSpecificPalette () {

	

	
	 byte backgroundPalette = PackageID.blueBackground;









return  backgroundPalette;


}


public void LoadBackgroundResources () {

	

	 Package p = RessourcesManager.Get().GetGlobalBackground();

	(mpBackgroundViewport)= (mpBackgroundViewport).Cast((p).GetEntryPoint(RSRC_globalBackgroundPackage.globalBackgroundViewport_ID), null);
	mpBackgroundViewport.SetViewport(this);
	mpBackgroundViewport.SendToBack();
	(mpBottomSoftKeyBackgroundSprite)= (mpBottomSoftKeyBackgroundSprite).Cast((p).GetEntryPoint(RSRC_globalBackgroundPackage.bottomSoftkeyBackgroundSprite_ID), null);

	(mpTopSoftKeyBackgroundSprite)= (mpTopSoftKeyBackgroundSprite).Cast((p).GetEntryPoint(RSRC_globalBackgroundPackage.topSoftkeyBackgroundSprite_ID), null);

}


public final static byte kGameFlowNormal = 0;
public final static byte kGameFlowSinglePlayerLoading = 1;

public void UnstackAllScenes () {

	



while(GetCurrentScene() != null)
	{
		GetCurrentScene().Dismiss((((int)ConstantsEnglishPortrait.automaticDismissCommand == (int)ConstantsEnglishLandscape.automaticDismissCommand) ? Constants.automaticDismissCommand : ConstantsGetter.GetConstants().automaticDismissCommandFct()));
	}

}


public StringToolbox mStringToolbox;
public Viewport mpBackgroundViewport;
public Sprite mpBottomSoftKeyBackgroundSprite;
public Sprite mpTopSoftKeyBackgroundSprite;
public boolean mMustSave;
public int mState;
public boolean mClearSceneToMainMenuAtNextTransition;
public TetrisMediaPlayer mpTetrisMediaPlayer;
public byte mGameFlowState;
public boolean mBlackBackgroundOnNextHourglass;
public TetrisInputControler mInputControler;
public void OnScreenSizeChange () {
	
	ScreenManager.Get().OnScreenSizeChange();








































}


public void Save () {
 




 boolean saveFileReadSucceded = OnWriteFile();





}


public boolean OnWriteFile () {


	mFileManager.StartWritingProcess();
	return  mFileManager.IsIdle();

}


public boolean OnReadFile () {


	mFileManager.StartReadingProcess();
	return  mFileManager.IsIdle();

}


public boolean OnDeleteFile () {


	mFileManager.StartDeletingProcess();
	return  mFileManager.IsIdle();

}


public FileHandler GetSerializationFile () {


	return  mFileManager.GetFile();

}


public FileManager mFileManager;
public SerializableContainer mConfigFilesContainer;
public RessourcesManager mRessourcesManager;
public LanguageManager mLanguageManager;
public ScreenManager mScreenManager;
public boolean mHardwareOrientationChanging;
public void SetMustSave()
{
	SetMustSave(true);
}
public static void StartAndInitializeTimeSystem(TimeSystem pTimeSystem)
{
	StartAndInitializeTimeSystem(pTimeSystem, (int)0);
}
public static void FinalizeAndStopTimeSystem(TimeSystem pTimeSystem)
{
	FinalizeAndStopTimeSystem(pTimeSystem, (int)0);
}

/* One dimension array */
public static TetrisApp[] InstArrayTetrisApp(int size) {
	TetrisApp[] newArray = new TetrisApp[size];
	for(int i = 0 ; i < size ; i++) newArray[i] = new TetrisApp();
	return newArray; }

/* Two dimension array */
public static TetrisApp[][] InstArrayTetrisApp(int size1, int size2) {
	TetrisApp[][] newArray = new TetrisApp[size1][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new TetrisApp[size2];
		for(int j = 0 ; j < size2 ; j++) newArray[i][j] = new TetrisApp(); }
	return newArray; }

/* Three dimension array */
public static TetrisApp[][][] InstArrayTetrisApp(int size1, int size2, int size3) {
	TetrisApp[][][] newArray = new TetrisApp[size1][][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new TetrisApp[size2][];
		for(int j = 0 ; j < size2 ; j++) {
			newArray[i][j] = new TetrisApp[size3];
			for(int k = 0 ; k < size3 ; k++) newArray[i][j][k] = new TetrisApp(); } }
	return newArray; }



}

