package ca.jamdat.tetris;

import ca.jamdat.flight.*;
import ca.jamdat.fuser.*;
import ca.jamdat.flight.String;
import ca.jamdat.flight.Package;


public class TetrisMediaPlayer {

public final static byte kSndLockDown = 0;
public final static byte kSndHardDrop = 1;
public final static byte kSndHold = 2;
public final static byte kSndLineClear1 = 3;
public final static byte kSndLineClear2 = 4;
public final static byte kSndLineClear3 = 5;
public final static byte kSndLineClear4 = 6;
public final static byte kSndLineClear1TSpin = 7;
public final static byte kSndLineClear2TSpin = 8;
public final static byte kSndSpecialAttack = 9;
public final static byte kSndGameOverLoose = 10;
public final static byte kSndGameOverWin = 11;
public final static byte kSndLevelUp = 12;
public final static byte kSndLineAttack = 13;
public final static byte kSndLineAttackOpponent = 14;
public final static byte kSndInGameMusic = 15;
public final static byte kSndInGameMusicSample1 = 16;
public final static byte kSndInGameMusicSample2 = 17;
public final static byte kSndGameSoundCount = 18;
public final static byte kSndIntro = 19;
public final static byte kSndNull = 20;


public  TetrisMediaPlayer () {
mMusicMode = (InGameMusicMode.kInGameMusicOff);






	
	mpGameMusicPlayer = SoundManager.Get().GetChannelSoundPlayer(0);
	mpSoundPlayer = SoundManager.Get().GetChannelSoundPlayer(1);


UnloadGameSounds();


	LoadIntroSound();




}


public void UnloadResources () {



if(mpSoundPlayer != null)
	{
		mpSoundPlayer.SetSound(null);
	}


	if(mpGameMusicPlayer != null)
	{
		mpGameMusicPlayer.SetSound(null);
	}

}


public void PlaySound ( byte snd, boolean forcePlaySound) {





if(!mSoundEnabled && !forcePlaySound)
	{
		return ;
	}

	if(snd < kSndGameSoundCount)
	{

		PlaySound(mpTetrisSounds[snd]);
	}

	else if(snd == kSndIntro)
	{


		

		if(mMusicMode != InGameMusicMode.kInGameMusicOff)
{

			PlaySound(mpIntroMusic);
		}
	}

}


public void StopSound () {






mpSoundPlayer.Stop();

}


public void PlayClearLineSnd (int clearedCount, boolean tSpin) {




	 byte clearSnd = kSndNull;


	switch(clearedCount)
	{
	case 1 : clearSnd = (tSpin ? kSndLineClear1TSpin : kSndLineClear1); break;
	case 2 : clearSnd = (tSpin ? kSndLineClear2TSpin : kSndLineClear2); break;
	case 3 : clearSnd = kSndLineClear3; break;
	case 4 : clearSnd = kSndLineClear4; break;
	default :

	}



    PlaySound(clearSnd);

}


public void PlayEndGameSound (boolean win) {

	PlaySound(win ? kSndGameOverWin : kSndGameOverLoose);

}


public void LoadGameSounds () {


	if(mpTetrisSounds[0] == null) 
	{
		UnloadIntroSound();
		 Package soundPack = RessourcesManager.Get().GetPackage(PackageID.gameSound, false);
		for(int i = 0;i < kSndGameSoundCount;i ++)
		{
			(mpTetrisSounds[i])= (mpTetrisSounds[i]).Cast((soundPack).GetEntryPoint(i), null);
		}

		mMusicModeChanged = false;
}

}


public void LoadIntroSound () {


	if(mpIntroMusic == null) 
	{
		 Package soundPack = RessourcesManager.Get().GetPackage(PackageID.introSound, false);
		(mpIntroMusic)= (mpIntroMusic).Cast((soundPack).GetEntryPoint(0), null);
	}

}


public void StartVibration (int milliseconds) {













if(mVibrationEnabled)
	{

		VibrationManager.Get().Start(milliseconds);














}

}


public void StopVibration () {


	VibrationManager.Get().Stop();










}


public void ReloadGameSounds() {
 UnloadGameSounds(); LoadGameSounds(); 
}


public void PlaySong (byte song, boolean sample) {



	

	if(mMusicMode == InGameMusicMode.kInGameMusicOff)
	{
		return ;
	}






mpGameMusicPlayer.Stop();

	
	mpGameMusicPlayer.SetLooping(!sample);

	mpGameMusicPlayer.SetSound(mpTetrisSounds[song]);
	mpGameMusicPlayer.Play();

}


public void StopInGameMusic () {

	mpGameMusicPlayer.Stop();

}


public void InitializeInGameMusic() {
 if(mMusicModeChanged) ReloadGameSounds(); 
}


public void UnloadGameSounds () {



	StopSound();


	mpGameMusicPlayer.SetSound(null);
RemoveSoundFromPlayer();
	
	for(int i = 0;i < kSndGameSoundCount;i ++)
	{
		mpTetrisSounds[i] = null;
	}
	RessourcesManager.Get().RemovePackageFromCache(PackageID.gameSound);

}


public void UnloadIntroSound () {


	RemoveSoundFromPlayer();

	mpIntroMusic = null;
	RessourcesManager.Get().RemovePackageFromCache(PackageID.introSound);

}


public void SetSoundEnabled(boolean soundEnabled) {
 mSoundEnabled = soundEnabled; 
}


public boolean IsSoundEnabled() {
 return  mSoundEnabled; 
}


public void SetVibrationEnabled(boolean vibrationEnabled) {
 mVibrationEnabled = vibrationEnabled; 
}


public void SetVolumeLevel (short volumeLevel) {










}


public void SetInGameMusicMode (byte musicMode) {


	mMusicModeChanged = (mMusicMode != musicMode && musicMode != InGameMusicMode.kInGameMusicOff);
mMusicMode = musicMode;

}


public static boolean IsGameMusic(byte snd) {
 return  snd == kSndInGameMusic || snd == kSndInGameMusicSample1 ||	snd == kSndInGameMusicSample2;  
}


public void RemoveSoundFromPlayer () {






mpSoundPlayer.SetSound(null);

}


public void PlaySound ( Sound pSound) {






if(pSound != null)
	{
		mpSoundPlayer.SetSound(pSound);


mpSoundPlayer.Play();
	}

}


public static TetrisApp GetApp () {

	return  TetrisApp.Get();

}


public SoundPlayer mpSoundPlayer;
public SoundPlayer mpGameMusicPlayer;
public boolean mMusicModeChanged;
public Sound[] mpTetrisSounds = new Sound[kSndGameSoundCount];

public Sound mpIntroMusic;
public boolean mSoundEnabled;
public boolean mVibrationEnabled;
public short mVolumeLevel;
public byte mMusicMode;
public void PlaySound(byte snd)
{
	PlaySound(snd, false);
}
public void PlaySong(byte song)
{
	PlaySong(song, false);
}

/* One dimension array */
public static TetrisMediaPlayer[] InstArrayTetrisMediaPlayer(int size) {
	TetrisMediaPlayer[] newArray = new TetrisMediaPlayer[size];
	for(int i = 0 ; i < size ; i++) newArray[i] = new TetrisMediaPlayer();
	return newArray; }

/* Two dimension array */
public static TetrisMediaPlayer[][] InstArrayTetrisMediaPlayer(int size1, int size2) {
	TetrisMediaPlayer[][] newArray = new TetrisMediaPlayer[size1][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new TetrisMediaPlayer[size2];
		for(int j = 0 ; j < size2 ; j++) newArray[i][j] = new TetrisMediaPlayer(); }
	return newArray; }

/* Three dimension array */
public static TetrisMediaPlayer[][][] InstArrayTetrisMediaPlayer(int size1, int size2, int size3) {
	TetrisMediaPlayer[][][] newArray = new TetrisMediaPlayer[size1][][];
	for(int i = 0 ; i < size1 ; i++) {
		newArray[i] = new TetrisMediaPlayer[size2][];
		for(int j = 0 ; j < size2 ; j++) {
			newArray[i][j] = new TetrisMediaPlayer[size3];
			for(int k = 0 ; k < size3 ; k++) newArray[i][j][k] = new TetrisMediaPlayer(); } }
	return newArray; }



}

